import path from 'path';
import fs from 'fs';
import { storage, baseURL } from '../query.ts';
import { chalk } from '../chalk.ts';
import { Result } from '@kevisual/query';
type DownloadTask = {
  downloadPath: string;
  downloadUrl: string;
  user: string;
  key: string;
  version: string;
};
export type Package = {
  id: string;
  name?: string;
  version?: string;
  description?: string;
  title?: string;
  user?: string;
  key?: string;
  [key: string]: any;
};
type Options = {
  check?: boolean;
  returnContent?: boolean;
  setToken?: boolean;
  hash?: string;
  [key: string]: any;
};
export const fetchLink = async (url: string, opts?: Options) => {
  const token = process.env.KEVISUAL_TOKEN || storage.getItem('token');
  const fetchURL = new URL(url);
  const check = opts?.check ?? false;
  const setToken = opts?.setToken ?? true;
  if (check) {
    if (!url.startsWith(baseURL)) {
      throw new Error('url must start with ' + baseURL);
    }
  }
  if (token && setToken) {
    fetchURL.searchParams.set('token', token);
  }
  if (opts?.hash) {
    fetchURL.searchParams.set('hash', opts.hash);
  }
  fetchURL.searchParams.set('download', 'true');

  const res = await fetch(fetchURL.toString());
  const blob = await res.blob();
  const type = blob.type;
  let content: Buffer | undefined;
  if (opts?.returnContent) {
    content = Buffer.from(await blob.arrayBuffer());
  }
  const pathname = fetchURL.pathname;
  const filename = pathname.split('/').pop();
  return {
    status: res.status,
    filename,
    blob,
    type,
    content,
  };
};
type InstallAppOpts = {
  appDir?: string;
  kevisualUrl?: string;
  /**
   * 是否是客户端, 下载到 assistant-config的下面
   */
  isClient?: boolean;
  /**
   * 是否是web, 下载到 web-config的下面
   */
  appType?: 'app' | 'web';
};
export const installApp = async (app: Package, opts: InstallAppOpts = {}) => {
  // const _app = demoData;
  const { appDir = '', kevisualUrl = 'https://kevisual.cn', isClient = false, appType = 'web' } = opts;
  const _app = app;
  try {
    let files = _app.data.files || [];
    const version = _app.version;
    let hasPackage = false;
    const user = _app.user;
    const key = _app.key;
    const packagePath = path.join(appDir, appType === 'app' ? 'package.json' : `${user}/${key}/package.json`);
    const downFiles = files
      .filter((file: any) => file?.path)
      .map((file: any) => {
        const name = file?.name || '';
        if (name.startsWith('package.json')) {
          hasPackage = true;
        }
        const noVersionPath = file.path.replace(`/${version}`, '');
        let downloadPath = noVersionPath;
        let downloadUrl = '';
        if (appType === 'app') {
          downloadPath = noVersionPath.replace(`${user}/${key}/`, '');
        }
        if (file.path.startsWith('http')) {
          downloadUrl = file.path;
        } else {
          downloadUrl = `${kevisualUrl}/${noVersionPath}`;
        }
        return {
          ...file,
          downloadPath: path.join(appDir, downloadPath),
          downloadUrl: downloadUrl,
        };
      });
    const downloadTasks: DownloadTask[] = downFiles as any;
    console.log('downloadTasks', downloadTasks);
    if (!hasPackage) {
      console.log('没有package.json文件, 生成一个');
      const pkg = { ..._app };
      if (pkg.data) {
        delete pkg.data.permission;
      }
      fs.writeFileSync(packagePath, JSON.stringify(pkg, null, 2));
    }
    // return;
    for (const file of downloadTasks) {
      const downloadPath = file.downloadPath;
      const downloadUrl = file.downloadUrl;
      const dir = path.dirname(downloadPath);
      if (!fs.existsSync(dir)) {
        fs.mkdirSync(dir, { recursive: true });
      }
      console.log('downloadUrwl', downloadUrl);
      const { blob, type } = await fetchLink(downloadUrl);
      if (type.includes('text/html')) {
        const html = await blob.text();
        if (html === 'fetchRes is error') {
          console.log(chalk.red('fetchRes is error'), '下载失败', downloadUrl);
          throw new Error('fetchRes is error');
        }
      }
      fs.writeFileSync(downloadPath, Buffer.from(await blob.arrayBuffer()));
    }
    let indexHtml = files.find((file: any) => file.name === 'index.html');
    // if (!indexHtml) {
    //   files.push({
    //     name: 'index.html',
    //     path: `${user}/${key}/index.html`,
    //   });
    //   fs.writeFileSync(path.join(appDir, `${user}/${key}/index.html`), JSON.stringify(app, null, 2));
    // }
    _app.data.files = files;
    return {
      code: 200,
      data: _app,
      message: 'Install app success',
    };
  } catch (error) {
    console.error(error);
    return {
      code: 500,
      message: 'Install app failed',
    };
  }
};
/**
 * 检查是否为空，如果为空则删除
 * @param appDir
 */
export const checkAppDir = (appDir: string) => {
  try {
    const files = fs.readdirSync(appDir);
    if (files.length === 0) {
      fs.rmSync(appDir, { recursive: true });
    }
  } catch (error) {}
};
export const checkFileExists = (path: string) => {
  try {
    fs.accessSync(path);
    return true;
  } catch (error) {
    return false;
  }
};
type UninstallAppOpts = {
  appDir?: string;
  type?: 'app' | 'web';
};
export const uninstallApp = async (app: Partial<Package>, opts: UninstallAppOpts = {}) => {
  const { appDir = '', type = 'web' } = opts;
  try {
    const { user, key } = app;
    const keyDir = path.join(appDir, user, key);
    const parentDir = path.join(appDir, user);
    if (!checkFileExists(appDir) || !checkFileExists(keyDir)) {
      return {
        code: 200,
        message: 'uninstall app success',
      };
    }
    try {
      // 删除appDir和文件
      fs.rmSync(keyDir, { recursive: true });
    } catch (error) {
      console.error(error);
    }
    type === 'web' && checkAppDir(parentDir);
    return {
      code: 200,
      message: 'Uninstall app success',
    };
  } catch (error) {
    console.error(error);
    return {
      code: 500,
      message: 'Uninstall app failed',
    };
  }
};

export type AiList = {
  name?: string;
  lastModified?: string;
  etag?: string;
  size?: number;
  path: string;
  pathname?: string;
  url?: string;
};
export const fetchAiList = async (url: string, opts?: { recursive: boolean }): Promise<Result<AiList[]>> => {
  const token = process.env.KEVISUAL_TOKEN || storage.getItem('token');
  const _url = new URL(url);
  const dir = _url.searchParams.get('dir');
  if (!dir) {
    _url.searchParams.set('dir', 'true');
  }
  if (opts?.recursive) {
    _url.searchParams.set('recursive', 'true');
  }
  if (!_url.pathname.endsWith('/')) {
    _url.pathname += '/';
  }
  const res = await fetch(_url.toString(), {
    method: 'GET',
    headers: {
      Authorization: 'Bearer ' + token,
    },
  });
  const data = await res.json();
  return data;
};
